/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(ab8d0700-9577-11e3-a5e2-0800200c9a66)]
interface nsIFxAccountsUIGlue : nsISupports
{
  // Returns a Promise.
  jsval signInFlow();

  // Returns a Promise.
  jsval refreshAuthentication(in DOMString accountId);
};
