/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIX509CertList;

[scriptable, uuid(1b75bdae-1757-4322-9d1e-cfcaa18cb710)]
interface nsINSSCertCache : nsISupports {

  /*
   * cacheAllCerts
   *
   * Creates a cache of all certificates currently known to NSS.
   */
  void cacheAllCerts();
  void cacheCertList(in nsIX509CertList list); 

  /*
   * get an X509CertList
   */
  nsIX509CertList getX509CachedCerts();

  /*
   * getCachedCerts
   *
   * Returns the cached CERTCertList*
   */
  [notxpcom, noscript] voidPtr getCachedCerts();
};

%{C++

#define NS_NSSCERTCACHE_CID { /* 3f429a14-dffe-417d-8cb8-fdf09bacd09e */ \
    0x3f429a14,                                                          \
    0xdffe,                                                              \
    0x417d,                                                              \
    {0x8c, 0xb8, 0xfd, 0xf0, 0x9b, 0xac, 0xd0, 0x9e}                     \
  }

#define NS_NSSCERTCACHE_CONTRACTID "@mozilla.org/security/nsscertcache;1"

%}
