/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMEventListener;

[scriptable, builtinclass, uuid(a09cdc35-6b1e-42ce-95bb-f8f10a354202)]
interface nsIDOMLockedFile : nsISupports
{
  readonly attribute nsISupports /* FileHandle */ fileHandle;

  // "readonly" or "readwrite"
  readonly attribute DOMString mode;

  readonly attribute boolean active;

  [implicit_jscontext]
  attribute jsval location;

  [implicit_jscontext]
  nsISupports /* FileRequest */
  getMetadata(/* DOMFileMetadataParameters */
              [optional /* all */] in jsval parameters);

  [implicit_jscontext]
  nsISupports /* FileRequest */
  readAsArrayBuffer(in unsigned long long size);

  nsISupports /* FileRequest */
  readAsText(in unsigned long long size,
             [optional] in DOMString encoding);

  [implicit_jscontext]
  nsISupports /* FileRequest */
  write(in jsval value);

  [implicit_jscontext]
  nsISupports /* FileRequest */
  append(in jsval value);

  [optional_argc]
  nsISupports /* FileRequest */
  truncate([optional] in unsigned long long size);

  nsISupports /* FileRequest */
  flush();

  void
  abort();

  [implicit_jscontext] attribute jsval oncomplete;

  [implicit_jscontext] attribute jsval onabort;

  [implicit_jscontext] attribute jsval onerror;
};
